// Checksum & CRC Support code

// Copyright 2023 Philip Koopman
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//    http://www.apache.org/licenses/LICENSE-2.0
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "support.h"

#ifdef VALIDATE  // fixed data word for validation
// Constant validation data array
void  InitializeData(uint8_t codeWordB[], uint32_t cwLenB)
{ 
  assert(cwLenB == 16);
  codeWordB[0] = '1';
  codeWordB[1] = '2';
  codeWordB[2] = '3';
  codeWordB[3] = '4';
  codeWordB[4] = '5';
  codeWordB[5] = '6';
  codeWordB[6] = '7';
  codeWordB[7] = '8';
  codeWordB[8] = '9';
  codeWordB[9] = 0;   // zero padding past data word length
  codeWordB[10] = 0;  // zero padding past data word length
  codeWordB[11] = 0;  // zero padding past data word length
  codeWordB[12] = 0;  // zero padding past data word length
  codeWordB[13] = 0;  // zero padding past data word length
  codeWordB[14] = 0;  // zero padding past data word length
  codeWordB[15] = 0;  // zero padding past data word length
}

void InitializeData16(uint8_t codeWordB[], uint32_t cwLenB, uint16_t codeWordW[], uint32_t cwLenW)
{ 
  InitializeData(codeWordB, cwLenB);
  assert(cwLenW == 5);
  codeWordW[0] = 0x3132 ; // '1' '2'
  codeWordW[1] = 0x3334 ; // '3' '4'
  codeWordW[2] = 0x3536 ; // '5' '6'
  codeWordW[3] = 0x3738 ; // '7' '8'
  codeWordW[4] = 0x0000; // initialized check value
}

#else // Initialize data word with pseudorandom data
void InitializeData(uint8_t codeWord[], uint32_t cwLen)
{ 
  uint32_t seed = (uint32_t) time(NULL);
  srand(seed); // random sequence selection based on time of day

  for(uint32_t index = 0; index < cwLen; index++)
  { // note that rand() is not suitable for large-scale experiments
    codeWord[index] = (uint8_t)(rand() & 0xFF);
  }
}

// Initialize a data word and zero the check value
void InitializeData16(uint8_t codeWord[], uint32_t dwLen, uint16_t codeWordW[], uint32_t cwLenW)
{ InitializeData(codeWord, dwLen);

  assert(cwLenW > 1);
  // repack the codeword8 as as 16-bit blocks
  uint32_t dwIndex = 0;
  for(uint32_t index = 0; index < cwLenW-1; index++)
  { codeWordW[index] = (uint16_t)(((uint32_t) codeWord[dwIndex++]) << 8);
    codeWordW[index] ^= (uint16_t) codeWord[dwIndex++];
  }
  codeWordW[cwLenW-1] = 0; // initialize check value
}

#endif

// Print a code word or data word
void PrintWord(uint8_t codeWord[], uint32_t cwLen)
{ 
  printf("Hex Word (bytes)=   ");
  for (uint32_t index = 0; index < cwLen; index++)
  { 
    printf("%02X ", codeWord[index]);
  }
  printf("\n");
}

// Print a code word or data word
void PrintWord16(uint16_t codeWordW[], uint32_t cwLenW)
{ 
  printf("Hex Word (16 bits)= ");
  for (uint32_t index = 0; index < cwLenW; index++)
  { 
    printf("%04X  ", codeWordW[index]);
  }
  printf("\n");
}